########################################################################################
# Load code files and data
########################################################################################
# 'Stats.R' contains all the basic functions for statistical analysis of microbiome data
# Change to the directory with the code/data files
setwd('~/Dropbox/Workspace/MayoClinic/2015_02_20_MWRR_new/Code/SubmissionCode')
source('Functions.R')           
load_package()
temp <- load('PairedSingle.RData')
temp
ind1 <- data.obj$meta.dat$Batch == '1'
ind2 <- data.obj$meta.dat$Batch == '2'

data.obj.raw <- data.obj
data.obj <- data.obj.rff
dist.obj <- dist.obj.rff

# Data sets for specific batches
data.obj1 <- subset_data(data.obj, ind1)
data.obj2 <- subset_data(data.obj, ind2)

dist.obj1 <- subset_dist(dist.obj, ind1)
dist.obj2 <- subset_dist(dist.obj, ind2)

meta.dat <- data.obj$meta.dat
beta.names <- c('UniFrac', 'WUniFrac')
alpha.names <- c('Observed', 'Shannon')
phenos <- c('Age', 'Sex', 'BMI', 'Race', 'Tobacco', 'Alcohol')

theme_set(theme_bw())

########################################################################################
# Sequencing data
########################################################################################
perform_sequence_stat_analysis(data.obj.raw)

########################################################################################
# Alpha_diversity
########################################################################################
set.seed(1234)
# Can be easily changed to investigate effect in each batch
ind <- ind1 | ind2

phylo.even <- rarefy_even_depth(phylo.obj)
x <- estimate_richness(phylo.even, measures=alpha.names)
cat('Alpha diversity analysis ... \n')
for (pheno in phenos) {
	cat("\nPhenotype: ", pheno, '\n')
	for (alpha.name in alpha.names) {
		mod1 <- aov(x[ind, alpha.name] ~ meta.dat[ind, 'Batch'] * meta.dat[ind, pheno])
		mod2 <- aov(x[ind, alpha.name] ~ meta.dat[ind, 'Batch'])
		pv1 <- anova(mod1, mod2)[2, 'Pr(>F)']
		cat(alpha.name, '  (Adjust Batch): p=', pv1, '\n')
	}
}


########################################################################################
# Beta_diversity analysis
########################################################################################
set.seed(1234)

cat('Beta diversity analysis .....\n')
pv.mat <- r.mat <- matrix(NA, length(beta.names), length(phenos), dimnames=list(beta.names, phenos))
cat("Permanova test results after adjusting for batch effects...\n\n")
for (pheno in phenos) {
	cat("Phenotype: ", pheno, '\n')
	Batch <- data.obj$meta.dat[, 'Batch']
	voi <- data.obj$meta.dat[, pheno]
	for (dist.name in beta.names) {
		x1 <- as.numeric(Batch) - 1
		if(is.factor(voi) == T) {
			x2 <- as.numeric(voi) - 1
		} else {
			x2 <- voi
		}
		x3 <- x1 * x2
		aov.tab <- adonis(as.dist(dist.obj[[dist.name]]) ~ Batch + cbind(x2, x3))$aov.tab
		pv <- aov.tab[2, 6]
		R2 <- aov.tab[2, 5]
		
		# Adjusted R2 to account for small sample sizes, here df=2
		N <- length(Batch)
		R2 <- 1 - (1 - R2) * (N - 1) / (N - 3)          

		cat(dist.name, ' Distance (Adjust Batch): p=', pv, '\n')
		cat(dist.name, ' Distance (Adjust Batch): R2=', R2, '\n')
		
		r.mat[dist.name, pheno] <- R2
		pv.mat[dist.name, pheno] <- pv
	} 
	cat("\n\n")
}

# Plot percent of explained varibility based on UniFrac
R2 <- r.mat['UniFrac', ]
R2 <- sort(R2, decr=T) * 100
res.df <- data.frame(Variable=names(R2), R2=R2, label=paste0(round(R2,2), "%"))
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")
res.df$Variable <- factor(res.df$Variable, levels=res.df$Variable) 
pdf(paste0('ExplainedVarianceByFactorUniFrac.pdf'), width=5, height=5)
obj <- ggplot(res.df, aes(x=Variable, y=R2, label=label)) + 
		geom_bar(stat='identity', position='dodge') +
		geom_bar(stat='identity', position='dodge', col='black', fill='gray', show_guide=FALSE,size=0.25) + 
		geom_text(aes(y=res.df$R2 + 0.05)) +
		xlab('') + 
		ylab('Percent Explained Variability') +
		theme_bw() +
		theme(legend.title=element_blank()) +
		theme(panel.border = element_rect(colour = "black")) +
		scale_fill_manual(values=as.character(cbPalette[c(2:7)]))
print(obj)
dev.off()


########################################################################################
# Differential abundance anlaysis 
########################################################################################
# We focus on taxa with prevalence > 10% and median nonzero proportion > 0.002
# Permutation tests were used to test for association, adjusting batches, when necessary
set.seed(1234)
cat("Perform differential abundance anlaysis ...\n")
prev <- 0.10
minp <- NULL
medianp <- 0.0002
diff.method <- 'perm'
fdr.cutoff <- 0.10   

# For BMI, Race, Tobacco, the batch interaction effects were not signficant. We therefore did not include batch interaction term in the model
diff.obj <- perform_differential_analysis(data.obj, grp.name='BMI',  method=diff.method, adj.name="Batch",
		cutoff=fdr.cutoff, prev=prev, minp=minp,  medianp=medianp, ann='BMI')
data.obj$meta.dat$BMIcat[data.obj$meta.dat$BMIcat == 'Overweight'] <- 'Normal'
data.obj$meta.dat$BMIcat <- factor(data.obj$meta.dat$BMIcat) 
visualize_differential_analysis(data.obj, diff.obj, grp.name='BMIcat', cutoff=fdr.cutoff, ann='BMI', hei1=7, wid1=8)

diff.obj <- perform_differential_analysis(data.obj, grp.name='Race',  method=diff.method, adj.name="Batch",
		cutoff=fdr.cutoff, prev=prev, minp=minp, medianp=medianp, ann='Race')
visualize_differential_analysis(data.obj, diff.obj, grp.name='Race', cutoff=fdr.cutoff, ann='Race', hei1=7, wid1=2.75)

diff.obj <- perform_differential_analysis(data.obj, grp.name='Tobacco',  method=diff.method, adj.name="Batch",
		cutoff=fdr.cutoff, prev=prev, minp=minp, medianp=medianp, ann='Tobacco')
visualize_differential_analysis(data.obj, diff.obj, grp.name='Tobacco', cutoff=fdr.cutoff, ann='Tobacco')

# For Sex and alcohol, the batch interaction effects were signficant. The signficance was from the 2nd batch for sex, and 1st batch for alcohol.
# We therefore conducted analysis in respective batches.
diff.obj <- perform_differential_analysis(data.obj2, grp.name='Sex',  method='wilcox', 
		cutoff=fdr.cutoff, prev=prev, minp=minp, medianp=medianp, ann='Sex')
visualize_differential_analysis(data.obj2, diff.obj, grp.name='Sex', cutoff=fdr.cutoff, ann='Sex', hei1=7, wid1=1.5)

diff.obj <- perform_differential_analysis(data.obj1, grp.name='Alcohol',  method='wilcox', 
		cutoff=fdr.cutoff, prev=prev, minp=minp, medianp=medianp, ann='Alcohol')
visualize_differential_analysis(data.obj1, diff.obj, grp.name='Alcohol', cutoff=fdr.cutoff, ann='Alcohol', hei1=7, wid1=2.75)

cat('Finished!\n')


